package com.landray.kmss.sys.attachment.restservice.wps.util;

import java.security.MessageDigest;
import org.apache.commons.lang3.StringUtils;
import com.landray.kmss.util.MD5Util;
import java.util.Arrays;
import com.landray.kmss.util.StringUtil;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.Key;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import org.apache.commons.codec.binary.Base64;
import java.util.Collections;
import java.util.Comparator;
import java.util.ArrayList;
import java.util.Map;

public class WpsUtil
{
    public static String getSignature(final Map<String, String> params, final String appSecret) throws Exception {
        final List<String> keys = new ArrayList<String>();
        for (final Map.Entry<String, String> entry : params.entrySet()) {
            keys.add(entry.getKey());
        }
        Collections.sort(keys, new Comparator<String>() {
            @Override
            public int compare(final String o1, final String o2) {
                return o1.compareTo(o2);
            }
        });
        final StringBuilder contents = new StringBuilder("");
        for (final String key : keys) {
            if ("_w_signature".equals(key)) {
                continue;
            }
            contents.append(String.valueOf(key) + "=").append(params.get(key));
        }
        contents.append("_w_secretkey=").append(appSecret);
        final byte[] bytes = genHMAC(contents.toString(), appSecret);
        String sign = new String(Base64.encodeBase64(bytes));
        try {
            sign = URLEncoder.encode(sign, "UTF-8");
        }
        catch (final UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return sign;
    }
    
    public static Map<String, String> paramToMap(final String paramStr) {
        final String[] params = paramStr.split("&");
        final Map<String, String> resMap = new HashMap<String, String>();
        for (int i = 0; i < params.length; ++i) {
            final String[] param = params[i].split("=");
            if (param.length >= 2) {
                final String key = param[0];
                String value = param[1];
                for (int j = 2; j < param.length; ++j) {
                    value = String.valueOf(value) + "=" + param[j];
                }
                resMap.put(key, value);
            }
        }
        return resMap;
    }
    
    public static byte[] genHMAC(final String data, final String key) throws Exception {
        try {
            final SecretKeySpec signinKey = new SecretKeySpec(key.getBytes("utf-8"), "HmacSHA1");
            final Mac mac = Mac.getInstance("HmacSHA1");
            mac.init(signinKey);
            final byte[] rawHmac = mac.doFinal(data.getBytes("utf-8"));
            return rawHmac;
        }
        catch (final NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        catch (final InvalidKeyException e2) {
            e2.printStackTrace();
        }
        return null;
    }
    
    public static boolean isWord(final String fileExt) {
        if (StringUtil.isNotNull(fileExt)) {
            String typeStr = "doc,dot,wps,wpt,docx,dotx,docm,dotm";
            typeStr = typeStr.replaceAll(" ", "");
            final String[] typeArr = typeStr.split(",");
            final List typeList = Arrays.asList(typeArr);
            if (typeList.contains(fileExt.toLowerCase())) {
                return true;
            }
        }
        return false;
    }
    
    public static boolean isExcel(final String fileExt) {
        if (StringUtil.isNotNull(fileExt)) {
            String typeStr = "xls,xlt,et,xlsx,xltx,csv,xlsm,xltm";
            typeStr = typeStr.replaceAll(" ", "");
            final String[] typeArr = typeStr.split(",");
            final List typeList = Arrays.asList(typeArr);
            if (typeList.contains(fileExt.toLowerCase())) {
                return true;
            }
        }
        return false;
    }
    
    public static boolean isPpt(final String fileExt) {
        if (StringUtil.isNotNull(fileExt)) {
            String typeStr = "ppt,pptx,pptm,ppsx,ppsm,pps,potx,potm,dpt";
            typeStr = typeStr.replaceAll(" ", "");
            final String[] typeArr = typeStr.split(",");
            final List typeList = Arrays.asList(typeArr);
            if (typeList.contains(fileExt.toLowerCase())) {
                return true;
            }
        }
        return false;
    }
    
    public static boolean isPdf(final String fileExt) {
        return StringUtil.isNotNull(fileExt) && ("pdf".equals(fileExt.toLowerCase()) || "ofd".equals(fileExt.toLowerCase()));
    }
    
    public static String sign(final String value, final String key) {
        final String signStr = String.valueOf(value) + "::" + key;
        return MD5Util.getMD5String(signStr);
    }
    
    public static String generateUserCtlSignature(final Map<String, String> map) {
        if (map == null) {
            return null;
        }
        final String userId = map.get("_w_userid");
        final String copy = map.get("_w_copy");
        final String export = map.get("_w_export");
        final String print = map.get("_w_print");
        final String value = String.valueOf(copy) + "##" + export + "##" + print;
        return sign(value, userId);
    }
    
    public static boolean checkSignature(final String value, final String key, final String signature) {
        final String sign = sign(value, key);
        return StringUtils.equals((CharSequence)sign, (CharSequence)signature);
    }
    
    public static String MD5_32(final String plainText) {
        String re_md5 = new String();
        try {
            final MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(plainText.getBytes());
            final byte[] b = md.digest();
            final StringBuffer buf = new StringBuffer("");
            for (int offset = 0; offset < b.length; ++offset) {
                int i = b[offset];
                if (i < 0) {
                    i += 256;
                }
                if (i < 16) {
                    buf.append("0");
                }
                buf.append(Integer.toHexString(i));
            }
            re_md5 = buf.toString();
        }
        catch (final Exception e) {
            e.printStackTrace();
        }
        return re_md5;
    }
}
